-- ==============================================================
-- FILE : USER_AP4.VHD  -  MUST be modified by the USER
-- ==============================================================
--
-- This file is part of Example 4, for the HERON-IO2V2 module.
--
-- This file must be modified by the user.
--
-- ==============================================================
--
--  Date        : 28/02/2002
--  Author      : R. Williams - HUNT ENGINEERING
--  Description : User code section
--
-- ==============================================================
--
--  Ver     Modified By      Date      Changes
--  ---     -----------      ----      -------
--  1.0     R. Williams    28-02-02    First Written
--  2.0     R. Williams    10-05-02    Adapted for the IO2V2 from
--                                     IO2V1.
--  2.1     R. Williams    10-10-02    New components added for
--                                     interfacing to all six HERON
--                                     input FIFOs and all six HERON
--                                     output FIFOs at the same time.
--                                     Added component HE_RD_6F in
--                                     place of component HE_RD_1F.
--                                     Added component HE_WR_6F in
--                                     place of component HE_WR_1F.
--  2.4     R. Williams    08-06-06    Removed HEPC8 support.
--                                     Minor fix to HE_RD_6F logic.
--
-- ==============================================================



-- ==============================================================
--   Package : CONFIG
-- ==============================================================
--
-- This package allow users to set up the different clock domains
-- of the application (FIFO, ADC interfaces) in the top level.
-- Please refer to the HUNT documentation for more details.
--
-- ==============================================================

package CONFIG is

  -- FIFO CLOCK
  -- ==========

--   -----------------------------------------------------------------
--  | FCLK_G_DOMAIN |  HIGH_FCLK_G  |  HIGH_FCLK_RD  |  HIGH_FCLK_WR  |
--   -----------------------------------------------------------------
--  |     True      | True / False  |     n.a.       |     n.a.       |  single FIFO clock
--   -----------------------------------------------------------------
--  |     False     |     n.a.      |  True / False  |  True / False  |  two FIFO clocks
--   -----------------------------------------------------------------

  -- To use a single common FIFO clock, set FCLK_G_DOMAIN = TRUE
  constant FCLK_G_DOMAIN : boolean := TRUE;

  -- When read/write FIFO clock frequency > 60 MHz, set HIGH_FCLK_G = TRUE
  -- This constant is used only when FCLK_G_DOMAIN = TRUE
  constant HIGH_FCLK_G   : boolean := TRUE;

  -- When read FIFO clock frequency > 60 MHz, set HIGH_FCLK_RD = TRUE
  -- This constant is used only when FCLK_G_DOMAIN = FALSE
  constant HIGH_FCLK_RD  : boolean := FALSE;

  -- When write FIFO clock frequency > 60 MHz, set HIGH_FCLK_WR = TRUE
  -- This constant is used only when FCLK_G_DOMAIN = FALSE
  constant HIGH_FCLK_WR  : boolean := FALSE;

  -- ADC CLOCK
  -- =========

  -- To drive ADC sample clock A from a signal inside the FPGA set INTERNAL_SCLK_A = TRUE
  -- to use an external sample clock, set INTERNAL_SCLK_A = FALSE
  constant INTERNAL_SCLK_A : boolean := TRUE;

  -- To use a single common ADC clock for A & B channels, set SCLK_G_DOMAIN = TRUE
  constant SCLK_G_DOMAIN : boolean := TRUE;

end CONFIG;


-- ==============================================================
--   Module : USER_AP
-- ==============================================================

library IEEE;
  use IEEE.std_logic_1164.ALL;
  use IEEE.numeric_std.all;
  use WORK.CONFIG.all;

--pragma translate_off
library sim_lib;
  use sim_lib.all;

library UNISIM;
  use UNISIM.VCOMPONENTS.all;
--pragma translate_on

entity USER_AP is
  port (
    RESET           : in  std_logic;                     -- asynchronous reset active high
    CONFIG          : in  std_logic;                     -- System wide Config signal
-- CLOCK IO
    OSC0            : in  std_logic;                     -- Xtal OSC0
    OSC1            : in  std_logic;                     -- Xtal OSC1
    OSC2            : in  std_logic;                     -- Xtal OSC2
    OSC3            : in  std_logic;                     -- Xtal OSC3 factory default fitted with 100Mhz
    CLKIN0          : in  std_logic;                     -- User clock input (unbuffered) 
    CLKIN1          : in  std_logic;                     -- User clock input (unbuffered)
    CLKI2           : in  std_logic;                     -- User clock input (buffered)
    CLKI3           : in  std_logic;                     -- User clock input (buffered)
    CLKOUT          : out std_logic;                     -- User clock output
    QTTL            : in  std_logic;
    DTTL            : out std_logic;
    PECL_CLK        : in  std_logic;
-- SERIAL IO
    R1OUT           : in  std_logic;
    R2OUT           : in  std_logic;
    T1IN            : out std_logic;
    T2IN            : out std_logic;
    RS485_232       : out std_logic;
    HDPLX           : out std_logic;
    FAST            : out std_logic;
-- MISCELLANEOUS IO
    UDPRES          : out std_logic;                     -- reset the carrier (high if NOT used)
    LED             : out std_logic_vector(4 downto 0);  -- LEDs
    ADDR_FLAGSEL    : in  std_logic;
    BOOTEN          : out std_logic;
    CID             : in  std_logic_vector(3 downto 0);
    MID             : in  std_logic_vector(3 downto 0);
-- UMIs
    UMI_IN          : in  std_logic_vector(3 downto 0);
    UMI_OUT         : out std_logic_vector(3 downto 0);
    UMI_EN          : out std_logic_vector(3 downto 0);
-- DIGITAL IO
    DIO_IN          : in  std_logic_vector(7 downto 0);
    DIO_OUT         : out std_logic_vector(7 downto 0);
    DIO_EN          : out std_logic_vector(7 downto 0);
-- READ FIFO CLK
    FCLK_RD         : in  std_logic;                     -- FIFO_IN clock to be used in this module (only when FCLK_G_DOMAIN=FALSE)
    SRC_FCLK_RD     : out std_logic;                     -- FIFO_IN clock source for the top level  (only when FCLK_G_DOMAIN=FALSE)
-- WRITE FIFO CLK
    FCLK_WR         : in  std_logic;                     -- FIFO_OUT clock to be used in this module (only when FCLK_G_DOMAIN=FALSE)
    SRC_FCLK_WR     : out std_logic;                     -- FIFO_OUT clock source for the top level  (only when FCLK_G_DOMAIN=FALSE)
-- COMMON FIFO CLK
    FCLK_G          : in  std_logic;                     -- Common FIFO clock to be used in this module (only when FCLK_G_DOMAIN=TRUE)
    SRC_FCLK_G      : out std_logic;                     -- Common FIFO clock source for the top level  (only when FCLK_G_DOMAIN=TRUE)
-- FIFO IN / HE_RD_6F interface
    INFIFO_READ_REQ : out std_logic_vector(5 downto 0);  -- FIFO_IN Read Request
    INFIFO_DVALID   : in  std_logic_vector(5 downto 0);  -- FIFO_IN Data Valid
    INFIFO_SINGLE   : in  std_logic_vector(5 downto 0);  -- FIFO_IN Single data item available
    INFIFO_BURST    : in  std_logic_vector(5 downto 0);  -- FIFO_IN Block of data available
    INFIFO0_D       : in  std_logic_vector(31 downto 0); -- FIFO_IN Data for FIFO 0
    INFIFO1_D       : in  std_logic_vector(31 downto 0); -- FIFO_IN Data for FIFO 1
    INFIFO2_D       : in  std_logic_vector(31 downto 0); -- FIFO_IN Data for FIFO 2
    INFIFO3_D       : in  std_logic_vector(31 downto 0); -- FIFO_IN Data for FIFO 3
    INFIFO4_D       : in  std_logic_vector(31 downto 0); -- FIFO_IN Data for FIFO 4
    INFIFO5_D       : in  std_logic_vector(31 downto 0); -- FIFO_IN Data for FIFO 5
-- FIFO OUT / HE_WR_6F interface
    OUTFIFO_READY   : in  std_logic_vector(5 downto 0);  -- FIFO_OUT Ready Flag
    OUTFIFO_WRITE   : out std_logic_vector(5 downto 0);  -- FIFO_OUT Write Command
    OUTFIFO_D       : out std_logic_vector(31 downto 0); -- FIFO_OUT Data
-- SERIAL BUS / HE_USER Message interface
    MSG_CLK         : out std_logic;                     -- HSB clock source for the top level
    MSG_DIN         : in  std_logic_vector(7 downto 0);  -- data received from HSB
    MSG_ADDR        : in  std_logic_vector(7 downto 0);  -- HSB address
    MSG_AEN         : in  std_logic;                     -- address enable
    MSG_WEN         : in  std_logic;                     -- write access
    MSG_REN         : in  std_logic;                     -- read access
    MSG_DONE        : in  std_logic;                     -- message successfully transmitted
    MSG_COUNT       : in  std_logic;                     -- user's counter enable
    MSG_CLEAR       : in  std_logic;                     -- user's counter clear
    MSG_READY       : out std_logic;                     -- to acknowledge an access
    MSG_SEND_MSG    : out std_logic;                     -- message send command
    MSG_CE          : out std_logic;                     -- to control speed operation
    MSG_DOUT        : out std_logic_vector(7 downto 0);  -- data sent to HSB
    MSG_SEND_ID     : out std_logic;                     -- ID information send command
    MSG_LAST_BYTE   : out std_logic;                     -- to indicate when the last byte is presented
-- ADC_A SCLK
    SCLK_A          : in  std_logic;                     -- ADC A clock to be used in this module (only when SCLK_G_DOMAIN=FALSE)
    SRC_SCLK_A      : out std_logic;                     -- ADC A clock source for the top level  (only when SCLK_G_DOMAIN=FALSE)
-- ADC_B SCLK
    SCLK_B          : in  std_logic;                     -- ADC B clock to be used in this module (only when SCLK_G_DOMAIN=FALSE)
    SRC_SCLK_B      : out std_logic;                     -- ADC B clock source for the top level  (only when SCLK_G_DOMAIN=FALSE)
-- GLOBAL ADC SCLK
    SCLK_G          : in  std_logic;                     -- Common ADC clock to be used in this module (only when SCLK_G_DOMAIN=TRUE)
    SRC_SCLK_G      : out std_logic;                     -- Common ADC clock source for the top level  (only when SCLK_G_DOMAIN=TRUE)
-- HE_ADC_A
    ADC_A           : in  std_logic_vector(11 downto 0); -- ADC_A data
    OTR_A           : in  std_logic;                     -- ADC_A over range flag
-- HE_ADC_B
    ADC_B           : in  std_logic_vector(11 downto 0); -- ADC_B data
    OTR_B           : in  std_logic;                     -- ADC_B over range flag
-- HE_DAC_A
    SCLK_DAC_A      : out std_logic;                     -- DAC_A clock source for the top level
    DAC_A           : out std_logic_vector(13 downto 0); -- DAC_A data
-- HE_DAC_B
    SCLK_DAC_B      : out std_logic;                     -- DAC_B clock source for the top level
    DAC_B           : out std_logic_vector(13 downto 0)  -- DAC_B data
  );
end USER_AP;


architecture EXAMPLE4 of USER_AP is

  signal MCLK     : std_logic;  -- Main clock
  signal LEDcount : unsigned(25 downto 0);

begin


  -- ==========================================================
  -- ==========================================================
  --       <<<<<<<   INSERT YOUR CODE HERE    >>>>>>>>>
  -- ==========================================================
  -- ==========================================================


  -------------------------------------------------------------
  -- OSC3 clock divided by 2

  MCLK <= OSC3;

  -------------------------------------------------------------
  -- Drive the UDP-RESET high (inactive)

  UDPRES <= '1';

  -------------------------------------------------------------
  -- Drive the BOOT ENABLE signal high (inactive)

  BOOTEN <= '1';

  -------------------------------------------------------------
  -- Drive unused signals

  DIO_OUT   <= (others=>'0');
  DIO_EN    <= (others=>'1');

  UMI_OUT   <= (others=>'0');
  UMI_EN    <= (others=>'1');

  CLKOUT    <= '0';

  DTTL      <= '0';
  T1IN      <= '0';
  T2IN      <= '0';
  RS485_232 <= '0';
  HDPLX     <= '0';
  FAST      <= '0';

  -------------------------------------------------------------
  -- FIFOs

  SRC_FCLK_RD <= '0';   -- clock source for READ FIFO
  SRC_FCLK_WR <= '0';   -- clock source for WRITE FIFO
  SRC_FCLK_G  <= MCLK;  -- global clock source for both FIFOs

  INFIFO_READ_REQ <= (others=>'0');
  OUTFIFO_WRITE   <= (others=>'0');

  OUTFIFO_D   <= (others=>'0');

  -------------------------------------------------------------
  -- Serial Bus
  
  MSG_CLK       <= OSC3;  -- clock for Message Interface
  MSG_READY     <= '1';
  MSG_SEND_MSG  <= '0';
  MSG_CE        <= '0';
  MSG_SEND_ID   <= '0';
  MSG_LAST_BYTE <= '0';
  MSG_DOUT      <= (others=>'0');

  -------------------------------------------------------------
  -- ADCs and DACs

  SRC_SCLK_A <= '0';  -- clock source for ADC_A
  SRC_SCLK_B <= '0';  -- clock source for ADC_B
  SRC_SCLK_G <= OSC3; -- global clock source for both ADCs and DACs

  SCLK_DAC_A <= SCLK_G;  -- clock of DAC_A
  SCLK_DAC_B <= SCLK_G;  -- clock of DAC_B

  DAC_A <= (NOT ADC_A(11)) & ADC_A(10 downto 0) & '0' & '0';
  DAC_B <= (NOT ADC_B(11)) & ADC_B(10 downto 0) & '0' & '0';

  -------------------------------------------------------------
  -- Led Flasher : dividing F_CLKG clock by 2**25

  process(RESET, FCLK_G)
  begin
    if RESET='1' then
      LEDcount <= (others=>'0');
    elsif rising_edge(FCLK_G) then
      LEDcount <= LEDcount + 1;
    end if;
  end process;

  LED(4) <= LEDcount(24);

  -- Drive LEDs high (inactive)
  LED(3 downto 0) <= (others=>'1');

end EXAMPLE4;

